
/*
	SQL Server Script 3: create the plane tables in dbo
 */

USE [CassandraTest]
GO

CREATE TABLE [dbo].[Plane](
	[PlaneID] [uniqueidentifier] NOT NULL PRIMARY KEY,
	[Maker] [sysname] NOT NULL,
	[Model] [varchar](75) NOT NULL)
GO

INSERT INTO Plane( PlaneID, Maker, Model ) 
VALUES
	( 'A71D6CD9-B4AC-4B6F-9EE7-DC65D3785E0C', 'Cessna', 'Denali' ),
	( '2AEBA76B-205E-4B01-BA7E-FD66323488D6', 'Bombadier', 'Challenger 350' ),
	( '4423D4D1-827A-4EB6-8619-EA77DCC330C6', 'Beechcraft', 'Baron G58' )

select * from dbo.Plane;

	/*----------*/

CREATE TABLE [dbo].[Member](
	[MemberID] [smallint] NOT NULL PRIMARY KEY,
	[Name] [sysname] NOT NULL,
	[Since] [smallint] NOT NULL,
	[CanFly] [bit] NOT NULL)

INSERT INTO dbo.Member( MemberID, Name, Since, CanFly )
VALUES
	( 1, 'Sue Wong', 2016, 1 ),
	( 2, 'Alan Poe', 2013, 1 ),
	( 3, 'Joe Topp', 2017, 0 )

select * from dbo.Member;

	/*----------*/

CREATE TABLE [dbo].[RegisteredCreditCard](
	[CreditCard] [char](16) NOT NULL PRIMARY KEY,
	[Member] [smallint] NOT NULL
		CONSTRAINT fk_RCC_Member REFERENCES dbo.Member(MemberID) );

INSERT INTO dbo.RegisteredCreditCard( CreditCard, Member )
VALUES
	( '0101020203030404', 1 ),
	( '0101020203030909', 1 ),
	( '0102030405060708', 3 ),
	( '0123456789012345', 3 ),
	( '7676767676767676', 2 ),
	( '0303030303030303', 3 )

select * from RegisteredCreditCard;

	/*----------*/

CREATE TABLE [dbo].[PlaneRental](
	[Plane] [uniqueidentifier] NOT NULL,
	[RentalDate] [date] NOT NULL,
	[Member] [smallint] NOT NULL,
	[MemberCreditCard] [char](16) NOT NULL,
 CONSTRAINT [PK_PlaneRental__plane_rentdt] PRIMARY KEY CLUSTERED 
(
	[Plane] ASC,
	[RentalDate] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
 CONSTRAINT [UQ_PlaneRental__mbr_rentdt] UNIQUE NONCLUSTERED 
(
	[Member] ASC,
	[RentalDate] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
 CONSTRAINT [UQ_PlaneRental__mcc_rentdt] UNIQUE NONCLUSTERED 
(
	[MemberCreditCard] ASC,
	[RentalDate] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[PlaneRental]  WITH CHECK ADD FOREIGN KEY([Member])
REFERENCES [dbo].[Member] ([MemberID])
GO

ALTER TABLE [dbo].[PlaneRental]  WITH CHECK ADD FOREIGN KEY([MemberCreditCard])
REFERENCES [dbo].[RegisteredCreditCard] ([CreditCard])
GO

ALTER TABLE [dbo].[PlaneRental]  WITH CHECK ADD FOREIGN KEY([Plane])
REFERENCES [dbo].[Plane] ([PlaneID])
GO

-- trigger 1: ensure credit card --> member IN SAME TABLE
CREATE TRIGGER [dbo].[trIU_PlaneRental__FD_cc_member]
	ON  [dbo].[PlaneRental]
	FOR INSERT, UPDATE
AS
BEGIN
	IF EXISTS
	( select	*
	  from		inserted ins
				JOIN
				dbo.PlaneRental prent
				ON ins.MemberCreditCard = prent.MemberCreditCard
	  where		ins.Member <> prent.Member )
	BEGIN ROLLBACK TRAN; PRINT '[dbo].[trIU_PlaneRental__FD_cc_member]' END
END
go

-- trigger 2: stronger form of trigger 1 - check with registered credit cards
CREATE TRIGGER [dbo].[trIU_PlaneRental__FD_cc_member_by_fkey]
	ON  [dbo].[PlaneRental]
	FOR INSERT, UPDATE
AS
BEGIN
	IF EXISTS
	( select	*
	  from		inserted ins
				JOIN
				dbo.RegisteredCreditCard rcc
				ON ins.MemberCreditCard = rcc.CreditCard
	  where		ins.Member <> rcc.Member )
	BEGIN ROLLBACK TRAN; PRINT '[dbo].[trIU_PlaneRental__FD_cc_member_by_fkey]' END
END
GO

-- trigger 3: ensure member is cleared to fly
CREATE TRIGGER [dbo].[trIU_PlaneRental__IC_canfly]
	ON	[dbo].[PlaneRental]
	FOR INSERT, UPDATE
AS
BEGIN
	IF EXISTS
	( select	*
	  from		inserted ins
				JOIN
				dbo.Member mbr
				ON ins.Member = mbr.MemberID
	  where		mbr.CanFly = 0 )
	BEGIN ROLLBACK TRAN; PRINT '[dbo].[trIU_PlaneRental__IC_canfly]' END
END
GO

INSERT INTO dbo.PlaneRental( Plane, RentalDate, Member, MemberCreditCard )
VALUES
	( '4423D4D1-827A-4EB6-8619-EA77DCC330C6', '2020-02-01', 2, '7676767676767676' ),
	( '2AEBA76B-205E-4B01-BA7E-FD66323488D6', '2020-01-17', 2, '7676767676767676' ),
	( 'A71D6CD9-B4AC-4B6F-9EE7-DC65D3785E0C', '2019-12-15', 1, '0101020203030404' )

select * from dbo.PlaneRental;
go

/*  tests: break each trigger */

-- trigger 1: credit card already used by different member
UPDATE	dbo.PlaneRental
SET		MemberCreditCard = '7676767676767676'
WHERE	Member = 1;
go

-- trigger 2: credit card registered to different person
INSERT INTO dbo.PlaneRental( Plane, RentalDate, Member, MemberCreditCard )
VALUES
	( '4423D4D1-827A-4EB6-8619-EA77DCC330C6', '2020-02-05', 2, '0303030303030303' )
go

-- trigger 3: member not cleared to fly
INSERT INTO dbo.PlaneRental( Plane, RentalDate, Member, MemberCreditCard )
VALUES
	( '4423D4D1-827A-4EB6-8619-EA77DCC330C6', '2020-01-04', 3, '0303030303030303' )
go

select * from dbo.PlaneRental;
